@@ -1,3 +1,18 @@
+# Copyright 2014 Tycho Andersen
+# Copyright 2014 Sean Vig
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#   http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 from __future__ import division, absolute_import
 
 __xcb_proto_version__ = "1.10"
@@ -261,7 +276,9 @@ class Extension(object):
         if key is None:
             self.c_key = ffi.NULL
         else:
-            self.c_key = key.to_cffi()
+            c_key = key.to_cffi()
+            cffi_explicit_lifetimes[self] = c_key
+            self.c_key = c_key
 
     def send_request(self, opcode, data, cookie=VoidCookie, reply=None,
                      is_checked=False):
@@ -275,16 +292,23 @@ class Extension(object):
         xcb_req.opcode = opcode
         xcb_req.isvoid = issubclass(cookie, VoidCookie)
 
-        xcb_parts = ffi.new("struct iovec[2]")
-        xcb_parts[0].iov_base = iov_base = ffi.new('char[]', data)
-        xcb_parts[0].iov_len = len(data)
-        xcb_parts[1].iov_base = ffi.NULL
-        xcb_parts[1].iov_len = -len(data) & 3  # is this really necessary?
-        cffi_explicit_lifetimes[xcb_parts] = iov_base
+        # XXX: send_request here will use the memory *before* the passed in
+        # xcb_parts pointer in some cases, so we need to allocate some for it
+        # to use, although we don't use it ourselves.
+        #
+        # http://lists.freedesktop.org/archives/xcb/2014-February/009307.html
+        xcb_parts = ffi.new("struct iovec[4]")
+
+        # Here we need this iov_base to keep this memory alive until the end of
+        # the function.
+        xcb_parts[2].iov_base = iov_base = ffi.new('char[]', data)  # noqa
+        xcb_parts[2].iov_len = len(data)
+        xcb_parts[3].iov_base = ffi.NULL
+        xcb_parts[3].iov_len = -len(data) & 3  # is this really necessary?
 
         flags = C.XCB_REQUEST_CHECKED if is_checked else 0
 
-        seq = self.conn.send_request(flags, xcb_parts, xcb_req)
+        seq = self.conn.send_request(flags, xcb_parts + 2, xcb_req)
 
         return cookie(self.conn, seq, is_checked)
 
@@ -378,11 +402,17 @@ class OffsetMap(object):
 
 class Connection(object):
 
+    """ `auth` here should be '<name>:<data>', a format bequeathed to us from
+    xpyb. """
     def __init__(self, display=None, fd=-1, auth=None):
         if auth is not None:
+            [name, data] = auth.split(six.b(':'))
+
             c_auth = ffi.new("xcb_auth_info_t *")
-            if C.xpyb_parse_auth(auth, len(auth), c_auth) < 0:
-                raise XcffibException("invalid xauth")
+            c_auth.name = ffi.new('char[]', name)
+            c_auth.namelen = len(name)
+            c_auth.data = ffi.new('char[]', data)
+            c_auth.datalen = len(data)
         else:
             c_auth = ffi.NULL
 
@@ -396,7 +426,7 @@ class Connection(object):
         if fd > 0:
             self._conn = C.xcb_connect_to_fd(fd, c_auth)
         elif c_auth != ffi.NULL:
-            self._conn = C.xcb_connect_to_display_with_auth(display, c_auth, i)
+            self._conn = C.xcb_connect_to_display_with_auth_info(display, c_auth, i)
         else:
             self._conn = C.xcb_connect(display, i)
         self.pref_screen = i[0]
